@@ -48,6 +48,8 @@ def cache_thing(prefix, cache_key, data, cond_dnfs, timeout, dbs=(), precall_key
     # Could have changed after last check, sometimes superficially
     if transaction_states.is_dirty(dbs):
         return
+    if prefix and precall_key == "":
+        precall_key = prefix
     load_script('cache_thing', settings.CACHEOPS_LRU)(
         keys=[prefix, cache_key, precall_key],
         args=[
@@ -493,6 +495,9 @@ class ManagerMixin(object):
         # NOTE: this will behave wrong if someone changed object fields
         #       before deletion (why anyone will do that?)
         invalidate_obj(instance, using=using)
+        # NOTE: this is needed because m2m_changed is not sent on such deletion:
+        #       https://code.djangoproject.com/ticket/17688
+        invalidate_m2o(sender, instance, using)
 
     def inplace(self):
         return self.get_queryset().inplace()
@@ -520,6 +525,16 @@ def invalidate_o2o(sender, old, instance, using=DEFAULT_DB_ALIAS):
             invalidate_dict(rmodel, {rfield: value}, using=using)
 
 
+def invalidate_m2o(sender, instance, using=DEFAULT_DB_ALIAS):
+    """Invoke invalidation for m2o and m2m queries to a deleted instance"""
+    all_fields = sender._meta.get_fields(include_hidden=True, include_parents=True)
+    m2o_fields = [f for f in all_fields if isinstance(f, models.ManyToOneRel)]
+    for f in m2o_fields:
+        value = getattr(instance, f.field_name)
+        rmodel, rfield = f.related_model, f.remote_field.attname
+        invalidate_dict(rmodel, {rfield: value}, using=using)
+
+
 def invalidate_m2m(sender=None, instance=None, model=None, action=None, pk_set=None, reverse=None,
                    using=DEFAULT_DB_ALIAS, **kwargs):
     """
